<?php

namespace Drupal\field_zip_file\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\field_zip_file\Plugin\Field\FieldType\ZipFileItem;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Formatter that shows the file size in a human readable way.
 *
 * @FieldFormatter(
 *   id = "zip_file_base",
 *   label = @Translation("Text"),
 *   field_types = {
 *     "zip_file"
 *   }
 * )
 */
class ZipFileBase extends FormatterBase {

  /**
   * File url generator object.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;

  /**
   * Constructs new ZipFileBase object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $fileUrlGenerator
   *   File url generator object.
   */
  public function __construct($plugin_id,
                              $plugin_definition,
                              FieldDefinitionInterface $field_definition,
                              array $settings,
                              $label,
                              $view_mode,
                              array $third_party_settings,
                              FileUrlGeneratorInterface $fileUrlGenerator
                              ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->fileUrlGenerator = $fileUrlGenerator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('file_url_generator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $options = parent::defaultSettings();

    $options['default_index'] = NULL;
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    $form['default_index'] = [
      '#type' => 'textfield',
      '#title' => $this->t('The default file to open.'),
      '#default_value' => $this->getSetting('default_index'),
      '#description' => $this->t('Sample: index.html. You can leave this empty if your server has configurations to load default index page.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    if ($this->getSetting('default_index')) {
      $summary[] = $this->t('Default index page: @index', ['@index' => $this->getSetting('default_index')]);
    }
    return $summary;
  }

  /**
   * Get Url of zip file.
   */
  protected function getUrl(ZipFileItem $zip_file) {
    $url = NULL;
    if (($file = $zip_file->get('entity')->getValue()) && $file_info = _field_zip_file($file)) {
      $url = $this->fileUrlGenerator->generateAbsoluteString($file_info['folder_uri']);
    }
    if ($this->getSetting('default_index')) {
      $url = $url . '/' . $this->getSetting('default_index');
    }

    return $url;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    foreach ($items as $delta => $item) {
      $elements[$delta] = ['#markup' => $this->getUrl($item)];
    }

    return $elements;
  }

}
