<?php

namespace Drupal\field_zip_file\Plugin\Field\FieldType;

use Drupal\file\Plugin\Field\FieldType\FileItem;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Plugin implementation of the 'zip file' field type.
 *
 * @FieldType(
 *   id = "zip_file",
 *   label = @Translation("Zip File"),
 *   description = @Translation("This field stores the ID of an zip file as an integer value."),
 *   category = @Translation("Reference"),
 *   default_widget = "zip_file_widget",
 *   default_formatter = "zip_file_html",
 *   list_class = "\Drupal\file\Plugin\Field\FieldType\FileFieldItemList",
 *   constraints = {"ReferenceAccess" = {}, "FileValidation" = {}}
 * )
 */
class ZipFileItem extends FileItem {
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    return [
      'file_extensions' => 'zip',
      'file_directory' => '[date:custom:Y]-[date:custom:m]',
      'max_filesize' => '',
      'description_field' => 0,
      'tracking_script_field' => 0,
      'tracking_script_field_default' => NULL,
    ] + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    $schema = parent::schema($field_definition);
    $schema['columns']['tracking_script'] = [
      'description' => 'Tracking script',
      'type' => 'text',
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties = parent::propertyDefinitions($field_definition);

    $properties['tracking_script'] = DataDefinition::create('string')
      ->setLabel(t('Tracking script'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::fieldSettingsForm($form, $form_state);

    $settings = $this->getSettings();

    $element['tracking_script_field'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable <em>Tracking Script</em> field'),
      '#default_value' => $settings['tracking_script_field'] ?? '',
      '#description' => $this->t('The tracking script field allows user to inject the tracking code/script to all html files'),
      '#weight' => 11,
      '#attributes' => [
        'data-name' => 'tracking_script_field',
      ],
    ];

    $element['tracking_script_field_default'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Default <em>Tracking Script</em>'),
      '#default_value' => $settings['tracking_script_field_default'] ?? '',
      '#description' => [
        '#theme' => 'token_tree_link',
        '#token_types' => 'all',
        '#prefix' => $this->t('Tracking script default value.'),
      ],
      '#weight' => 11,
      '#states' => [
        'visible' => [
          ':input[data-name="tracking_script_field"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave() {
    parent::preSave();

    $this->processFile();
  }

  /**
   * {@inheritdoc}
   */
  public function unzip() {
    if ($fieldArr = $this->getValue()) {
      if (isset($fieldArr['fids']) && $fids = $fieldArr['fids']) {
        foreach ($fids as $fid) {
          $file = \Drupal::entityTypeManager()->getStorage('file')->load($fid);
          $this->processFileEntity($file);
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function processFileEntity($file) {
    if ($file_info = _field_zip_file($file)) {
      $zipFile = $file_info['zip_file'];
      $folderUri = $file_info['folder_uri'];
      $zipFile->extract($folderUri);

      $fs = \Drupal::service('file_system');

      $tracking_script = $folderUri . '/tracking-script.txt';
      $fs->delete($tracking_script);

      if ($backup_html_files = $fs->scanDirectory($folderUri, '/\b(\.old|\.OLD)\b/')) {
        foreach ($backup_html_files as $file) {
          $fs->move($file->uri, $folderUri . '/' . $file->name . '.html', FileSystemInterface::EXISTS_REPLACE);
        }
      }

      if ($this->tracking_script) {
        $fs->saveData($this->tracking_script, $tracking_script, FileSystemInterface::EXISTS_REPLACE);
        if ($html_files = $fs->scanDirectory($folderUri, '/\b(\.html|\.HTML)\b/')) {
          foreach ($html_files as $file) {
            $fs->copy($file->uri, $folderUri . '/' . $file->name . '.old', FileSystemInterface::EXISTS_REPLACE);
            $this->insertTrackingScript($file->uri, $this->tracking_script);
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function processFile() {
    if ($file = $this->get('entity')->getValue()) {
      $this->processFileEntity($file);
    }
  }

  /**
   * Re-write html file with the tracking script.
   */
  protected function insertTrackingScript($file, $tracking_script) {
    $token_service = \Drupal::token();

    $entity = $this->parent->parent->getEntity();

    $token_data = [];
    $token_data[$entity->getEntityType()->id()] = $entity;

    $token_options = ['clear' => TRUE];

    $original = file_get_contents($file);
    $lines = [];

    $tracking_script_replaced = $token_service->replace($tracking_script, $token_data, $token_options);

    foreach (explode("\n", $original) as $line) {
      if (strstr($line, '</head>')) {
        $lines[] = str_replace('</head>', $tracking_script_replaced . "\r\n</head>", $line);
      }
      else {
        $lines[] = $line;
      }
    }
    $new = implode("\n", $lines);
    file_put_contents($file, $new);
  }

}
