<?php

namespace Drupal\field_zip_file\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\file\Plugin\Field\FieldWidget\FileWidget;

/**
 * Plugin implementation of the 'zip_file_widget' widget.
 *
 * @FieldWidget(
 *   id = "zip_file_widget",
 *   label = @Translation("Zip File"),
 *   field_types = {
 *     "zip_file"
 *   }
 * )
 */
class ZipFileWidget extends FileWidget {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'hide_unzipped_url' => FALSE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['hide_unzipped_url'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Hide the URL of the unzipped folder'),
      '#default_value' => $this->getSetting('hide_unzipped_url'),
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $summary[] = $this->t('Hide unzipped URL: @hide_unzipped_url', ['@hide_unzipped_url' => $this->getSetting('hide_unzipped_url') ? $this->t('Yes') : $this->t('No')]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $field_settings = $this->getFieldSettings();

    // The field settings include defaults for the field type. However, this
    // widget is a base class for other widgets (e.g., ImageWidget) that may act
    // on field types without these expected settings.
    $field_settings += [
      'display_default' => NULL,
      'display_field' => NULL,
      'description_field' => NULL,
      'tracking_script_field' => NULL,
      'tracking_script_field_default' => NULL,
    ];

    $defaults = [
      'tracking_script' => $field_settings['tracking_script_field_default'],
    ];

    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $element['#tracking_script_field'] = $field_settings['tracking_script_field'];
    $element['#default_value'] = $element['#default_value'] + $defaults;

    if (empty($this->getSetting('hide_unzipped_url'))) {
      foreach ($items as $item) {
        $item->unzip();
      }
      $default_fids = $element['#extended'] ? $element['#default_value']['fids'] : $element['#default_value'];
      if (!empty($default_fids)) {
        $element['infos'] = [
          '#type' => 'container',
          '#weight' => 999,
        ];
        foreach ($default_fids as $fid) {
          if ($file = \Drupal::entityTypeManager()
            ->getStorage('file')
            ->load($fid)) {
            if ($file_info = _field_zip_file($file)) {
              $file_url = \Drupal::service('file_url_generator')->generateAbsoluteString($file_info['folder_uri']);
              $url = Url::fromUri($file_url, ['attributes' => ['target' => '_blank']]);
              $link = Link::fromTextAndUrl($url->toString(), $url)->toString();
              $element['infos'][] = [
                '#markup' => $this->t('Extracted files are located to: @path', ['@path' => $link]),
              ];
            }
          }
        }
      }
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function process($element, FormStateInterface $form_state, $form) {
    $element = parent::process($element, $form_state, $form);
    $item = $element['#value'];
    $item['fids'] = $element['fids']['#value'];

    if ($element['#tracking_script_field'] && $item['fids']) {
      $element['tracking_script'] = [
        '#type' => 'textarea',
        '#title' => new TranslatableMarkup('Tracking script'),
        '#value' => $item['tracking_script'] ?? $element['#default_value']['tracking_script'],
        '#description' => [
          '#theme' => 'token_tree_link',
          '#token_types' => 'all',
        ],
      ];
    }

    return $element;
  }

}
